#include "menu.h"
#include "libmy/key.h"
#include "libmy/gbfs.h"
#include "libmy/mode3.h"
#include "libmy/spr.h"

//---------------------------------------------------------------------------
ST_MENU Menu;


//---------------------------------------------------------------------------
EWRAM_CODE void MenuInit()
{
	_Memset(&Menu, 0x00, sizeof(ST_MENU));


	u16 arcCnt = GbfsGetArcCnt();			// BASA[JCu
	u16 scnCnt = Mode3GetScnCy() - 1;		// \ł鍀ڐB-1́A^Cgł

	if(arcCnt == 0)
	{
		SystemError("[Err] MenuInit ArcCnt == 0");
	}


	ST_MENU* p = &Menu;

	p->maxArcCnt  = arcCnt;
	p->maxScnCnt  = scnCnt;
	p->maxPageCnt = Div(arcCnt, scnCnt);
	p->maxSelCnt  = DivMod(arcCnt, scnCnt);

	if(p->maxSelCnt == 0)
	{
		p->maxPageCnt--;
		p->maxSelCnt = scnCnt;
	}

	Menu.state = MENU_LIST;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExec()
{
	switch(Menu.state)
	{
	case MENU_LIST:
		MenuExecList();
		break;

	case MENU_SEL:
		MenuExecSel();
		break;

	default:
		SystemError("[Err] MenuExec");
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecList()
{
	MenuDrawList();
	MenuDrawCursor();

	Menu.state = MENU_SEL;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuExecSel()
{
	ST_MENU* p = &Menu;

	// s[gL[擾܂
	u16 key = KeyGetRep();

	if(key & KEY_UP)
	{
		if(p->dispSel == 0 && p->dispPageCnt == 0)
		{
			return;
		}
		p->dispSel--;

		if(p->dispSel != -1)
		{
			MenuDrawCursor();
			return;
		}

		p->dispSel = p->maxScnCnt - 1;
		p->dispPageCnt--;
		p->state = MENU_LIST;
	}
	else if(key & KEY_DOWN)
	{
		if(p->dispSel == p->maxSelCnt-1 && p->dispPageCnt == p->maxPageCnt)
		{
			return;
		}
		p->dispSel++;

		if(p->dispSel != p->maxScnCnt)
		{
			MenuDrawCursor();
			return;
		}

		p->dispSel = 0;
		p->dispPageCnt++;
		p->state = MENU_LIST;
	}
	else if(key & KEY_A)
	{
		Menu.state = MENU_END;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuRestart()
{
	Menu.state = MENU_LIST;
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuDrawList()
{
	s16 st = Menu.dispPageCnt * Menu.maxScnCnt;
	s16 ed;

	if(Menu.dispPageCnt == Menu.maxPageCnt)
	{
		ed = st + Menu.maxSelCnt;
	}
	else
	{
		ed = st + Menu.maxScnCnt;
	}

	// ^Cg
	Mode3Cls();
	Mode3DrawStrLf("[[Tiny Basic 3 2008/12/16]]");

	// t@Cꗗ
	while(st < ed)
	{
		GbfsGetPointer2(st++);

		Mode3DrawStr(" ");						// J[\\̂߂̗]
		Mode3DrawStrLf(GbfsGetFileName());
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuDrawCursor()
{
	s16 cy = Mode3GetImgCy();
	SprSetCur(0, cy + cy * Menu.dispSel);
}
//---------------------------------------------------------------------------
EWRAM_CODE void MenuCls()
{
	Mode3Cls();
	SprSetCur(0, SCREEN_HEIGHT);
}
//---------------------------------------------------------------------------
EWRAM_CODE char* MenuGetFilePointer()
{
	u16 cnt = (Menu.dispPageCnt * Menu.maxScnCnt) + Menu.dispSel;

	return GbfsGetPointer2(cnt);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool MenuIsEnd()
{
	return (Menu.state == MENU_END) ? TRUE : FALSE;
}
